/*
 * Copyright (c) 2004 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: lua_context.c,v 1.5 2003/11/22 11:37:29 zinnia Exp $
 */

#include "sps.h"

#define LUA_FUNCTION_NAME(name) luafunction_##name
#define LUA_FUNCTION(name) static int LUA_FUNCTION_NAME(name) (lua_State* L)
#define REGISTER_LUA_FUNCTION(lua,name) lua_register(lua, "sps_" #name, LUA_FUNCTION_NAME(name))


LUA_FUNCTION(clear)
{
	SPS_Fill();
	return 0;
}

LUA_FUNCTION(pause)
{
	int timer = -1;
	if(lua_gettop(L) >= 1) {
		timer = lua_tonumber(L, 1);
	}

	SPS_Pause(timer);
	return 0;
}

LUA_FUNCTION(movecursor)
{
	if(lua_gettop(L) >= 2) {
		SPS_MoveCursor(lua_tonumber(L, 1), lua_tonumber(L, 2));
	}
	return 0;
}

LUA_FUNCTION(puttext)
{
	if(lua_gettop(L) >= 1){
		const char* fontindex = "0";
		SPS_VERTICAL_ALIGN align = SPS_VERTICAL_ALIGN_BOTTOM;

		if(lua_gettop(L) >= 2){
			fontindex = lua_tostring(L, 2);
		}
		if(lua_gettop(L) >= 3){
			align = SPS_Vertical_Align_Parse(lua_tostring(L, 3));
		}
		SPS_PutText(lua_tostring(L, 1), fontindex, align);
	}
	return 0;
}

LUA_FUNCTION(deffont)
{
	if(lua_gettop(L) >= 2){
		gpointer oldvalue, oldkey;
		const char* path;
		const char* name;

		name = lua_tostring(L, 1);
		path = lua_tostring(L, 2);

		if(g_hash_table_lookup_extended(context->fontpath, name, &oldkey, &oldvalue)){
			/* replace */
			g_hash_table_remove(context->fontpath, name);
			g_free(oldkey);
			g_free(oldvalue);
		} 
		g_hash_table_insert(context->fontpath, g_strdup(name), g_strdup(path));
		fprintf(stderr, "fontpath(%s) = %s\n", name, path);
	}
	return 0;
}

LUA_FUNCTION(deffontindex)
{
	if(lua_gettop(L) >= 3){
		char* oldkey; void* oldvalue;
		const char* name;
		const char* id;
		const char* path;
		int point;
		int aa = 0;
		TTF_Font* font;

		id = lua_tostring(L, 1);
		name = lua_tostring(L, 2);
		point = RatioToPixel(lua_tonumber(L, 3));
		if(lua_gettop(L) >= 4){
			aa = lua_tonumber(L, 4);
		}
		path = g_hash_table_lookup(context->fontpath, name);
		if(path != NULL){
			/* XXX point sanity check */
			font = TTF_OpenFont(path, point);
			if(font != NULL){
				gpointer oldkey, oldvalue;
				if(g_hash_table_lookup_extended(context->font, id, &oldkey, &oldvalue)){
					g_hash_table_remove(context->font, id);
					g_free(oldkey);
					TTF_CloseFont(oldvalue);
				}
				g_hash_table_insert(context->font, g_strdup(id), font);
				fprintf(stderr, "fontindex(%s) = %s at %d point\n", id, path, point);
			} else {
				fprintf(stderr, "TTF_OpenFont: %s\n", SDL_GetError());
			}
		} else {
			fprintf(stderr, "font name '%s' not found.\n", name);
		}
	}
	return 0;
}

LUA_FUNCTION(forecolor)
{
	if(lua_gettop(L) >= 1){
		SPS_ForeColor(lua_tostring(L, 1));
	}
	return 0;
}
LUA_FUNCTION(backcolor)
{
	if(lua_gettop(L) >= 1){
		SPS_BackColor(lua_tostring(L, 1));
	}
	return 0;
}

LUA_FUNCTION(puthbar)
{
	if(lua_gettop(L) >= 1)
	{
		SPS_PutHBar(lua_tonumber(L, 1));
	}
	return 0;
}

LUA_FUNCTION(moveto)
{
	if(lua_gettop(L) >= 2)
	{
		SPS_MoveTo(lua_tonumber(L, 1), lua_tonumber(L, 2));
	}
	return 0;

}
LUA_FUNCTION(beginline)
{
	const char* halign = "";
	if(lua_gettop(L) >= 1){
		halign = lua_tostring(L, 1);
	}
	SPS_BeginLine(halign);
	return 0;
}
LUA_FUNCTION(endline)
{
	SPS_EndLine();
	return 0;
}

LUA_FUNCTION(fontstyle)
{
	if(lua_gettop(L) >= 2){
		SPS_FontStyle(lua_tostring(L, 1), lua_tostring(L, 2));
	}
	return 0;
}

LUA_FUNCTION(hmargin)
{
	if(lua_gettop(L) >= 1){
		SPS_HMargin(lua_tonumber(L, 1));
	}
	return 0;
}
LUA_FUNCTION(deficon)
{
	if(lua_gettop(L) >= 2){
		SPS_DefIcon(lua_tostring(L, 1), lua_tostring(L, 2));
	}
	return 0;
}

LUA_FUNCTION(puticon)
{
	if(lua_gettop(L) >= 1){
		SPS_VERTICAL_ALIGN align = SPS_VERTICAL_ALIGN_BOTTOM;

		if(lua_gettop(L) >= 2){
			align = SPS_Vertical_Align_Parse(lua_tostring(L, 3));
		}
		SPS_PutIcon(lua_tostring(L, 1), align);
	}
	return 0;
}

LUA_FUNCTION(beginpage){ 
	SPS_BeginPage();
	return 0;
 }

LUA_FUNCTION(endpage){ return 0; }
void lua_context_prepare(lua_State* L)
{
	REGISTER_LUA_FUNCTION(L, clear);
	REGISTER_LUA_FUNCTION(L, pause);
	REGISTER_LUA_FUNCTION(L, movecursor);
	REGISTER_LUA_FUNCTION(L, puttext);

	REGISTER_LUA_FUNCTION(L, deffont);
	REGISTER_LUA_FUNCTION(L, deffontindex);

	REGISTER_LUA_FUNCTION(L, forecolor);
	REGISTER_LUA_FUNCTION(L, backcolor);

	REGISTER_LUA_FUNCTION(L, puthbar);
	REGISTER_LUA_FUNCTION(L, moveto);
	REGISTER_LUA_FUNCTION(L, beginline);
	REGISTER_LUA_FUNCTION(L, endline);
	REGISTER_LUA_FUNCTION(L, fontstyle);
	
	REGISTER_LUA_FUNCTION(L, hmargin);
	REGISTER_LUA_FUNCTION(L, deficon);
	REGISTER_LUA_FUNCTION(L, puticon);

	REGISTER_LUA_FUNCTION(L, beginpage);
	REGISTER_LUA_FUNCTION(L, endpage);
}
