/*
 * Copyright (c) 2004 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: sps.h,v 1.5 2003/11/22 11:37:29 zinnia Exp $
 */

#ifndef _SPS_H
#define _SPS_H
#include <stdio.h>
#include <stdlib.h>
#include "SDL.h"
#include "SDL_thread.h"

#include "SDL_image.h"

#include "SDL_ttf.h"

#include <lua.h>
#include <iconv.h>

#include <glib.h>
#define G_LIST_FOREACH(list,p) for(p = g_list_first(list); p != NULL; p = g_list_next(p))

#define DEFAULT_ENCODING "SJIS"

typedef enum {
	SPS_STATE_PAGEWAIT,
	SPS_STATE_PAUSE,
	SPS_STATE_DONE,
	SPS_STATE_DRAW,
	SPS_STATE_PAGELOAD,
} SPS_STATE;


typedef enum {
	SPS_VERTICAL_ALIGN_TOP,
	SPS_VERTICAL_ALIGN_CENTER,
	SPS_VERTICAL_ALIGN_BOTTOM,
} SPS_VERTICAL_ALIGN;

typedef enum {
	SPS_HORIZONTAL_ALIGN_LEFT,
	SPS_HORIZONTAL_ALIGN_CENTER,
	SPS_HORIZONTAL_ALIGN_RIGHT,
} SPS_HORIZONTAL_ALIGN;

typedef struct {
	lua_State* lua;          /* lua processor */
	Uint32 backcolor;        /* default bg color */
	Uint32 forecolor;	
	
	SDL_Surface* surface;    /* destination surface */
	const char* path;        /* path for source file */
	iconv_t iconv_context;

	int pageno;              /* page no */
	int maxpage;

	int in_linerender;       /* line rendering */

//	SPS_VERTICAL_ALIGN valign;
	SPS_HORIZONTAL_ALIGN halign;

	int hmargin;

	SDL_Rect cursor;         /* current cursor */

	SDL_Thread* thread;      /* processing thread */

	GList* pages;

	GHashTable* icons;       /* icon table: name -> surface */
	GHashTable* fontpath;    /* name -> font path */
	GHashTable* font;        /* no -> TTF_Font* */

	GList* line;

} SPS_Context;

typedef struct {
	Uint32 startticks;
	Uint32 frames;

	SPS_STATE state;
	int showfps;

	int redraw;

	int w, h;
	int fullscreen;

	SDL_mutex* mutex;
	SDL_Surface* display;

} SPS_Global;


typedef struct {

	gunichar* ucs4text;
	TTF_Font* font;
	SDL_Color color;
	int style;
	SPS_VERTICAL_ALIGN valign;
	int done;

	SDL_Surface* surface;

} SPS_Chunk_Context;

extern SPS_Global global;
extern SPS_Context* context;

extern void showfps();

extern void lua_context_prepare(lua_State* L);

extern SPS_VERTICAL_ALIGN SPS_Vertical_Align_Parse(const char* expression);
extern SPS_HORIZONTAL_ALIGN SPS_Horizontal_Align_Parse(const char* expression);

extern void SPS_Fill();

extern void SPS_Pause(int timer);
extern void SPS_MoveCursor(int x, int y);
extern void SPS_PutText(const char* text, const char* fontindex, SPS_VERTICAL_ALIGN align);

extern void SPS_ForeColor(const char* color);
extern void SPS_BackColor(const char* color);
extern void SPS_MoveTo(int x, int y);
extern void SPS_EndLine();
extern void SPS_BeginLine(const char* halign);
extern void SPS_FontStyle(const char* fontindex, const char* expression);
extern void SPS_HMargin(int m);

extern void SPS_DefIcon(const char* no, const char* filename);
extern void SPS_PutIcon(const char* iconindex, SPS_VERTICAL_ALIGN align);

extern void SPS_BeginPage();
#endif
